"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const Enums_1 = require("../Enums");
const services_1 = require("../services");
const PreviewImages_1 = require("./PreviewImages");
/**
 * Resets the document viewer state to the default.
 */
exports.resetDocumentData = () => ({
    type: 'SN_DOCVIEWER_DOCUMENT_RESET_DOCUMENT',
});
/**
 * Thunk action that polls document data from the specified API
 * @param hostName the host name, e.g. 'https://my-sensenet-site.com'
 * @param idOrPath Id or full path for the document, e.g.: 'Root/Sites/MySite/MyDocLib/('doc.docx')
 * @param version The document version
 */
exports.pollDocumentData = (hostName, idOrPath, version) => {
    return (dispatch, getState, api) => __awaiter(this, void 0, void 0, function* () {
        dispatch(exports.resetDocumentData());
        let docData;
        while (!docData || docData.pageCount === Enums_1.PreviewState.Loading) {
            try {
                docData = yield api.getDocumentData({ idOrPath, hostName });
                if (!docData || docData.pageCount === Enums_1.PreviewState.Loading) {
                    yield new Promise((resolve, reject) => setTimeout(() => { resolve(); }, getState().pollInterval));
                }
            }
            catch (error) {
                dispatch(exports.documentReceiveErrorAction(error || Error('Error loading document')));
                return;
            }
        }
        try {
            const [canEdit, canHideRedaction, canHideWatermark] = yield Promise.all([
                yield api.canEditDocument(docData),
                yield api.canHideRedaction(docData),
                yield api.canHideWatermark(docData),
            ]);
            dispatch(exports.documentPermissionsReceived(canEdit, canHideRedaction, canHideWatermark));
        }
        catch (error) {
            dispatch(exports.documentPermissionsReceived(false, false, false));
        }
        dispatch(exports.documentReceivedAction(docData));
        dispatch(PreviewImages_1.getAvailableImages(docData));
    });
};
/**
 * Action that updates the store with the received document data
 * @param document The received document data
 */
exports.documentReceivedAction = (document) => ({
    type: 'SN_DOCVIEWER_DOCUMENT_DATA_RECEIVED',
    document,
});
/**
 * Action that updates the store with a document receive error
 * @param error The error message
 */
exports.documentReceiveErrorAction = (error) => ({
    type: 'SN_DOCVIEWER_DOCUMENT_DATA_RECEIVE_ERROR',
    error,
});
/**
 * Action that updates a provided shape data
 * @param shapeType the type of the shape
 * @param shapeGuid the unique identifyer for the shape
 * @param shapeData the new shape data
 */
exports.updateShapeData = (shapeType, shapeGuid, shapeData) => ({
    type: 'SN_DOCVIEWER_DOCUMENT_UPDATE_SHAPE',
    shapeType,
    shapeGuid,
    shapeData,
});
/**
 * Action that removes a specified shape
 * @param shapeType The type of the shape
 * @param shapeGuid The unique identifier for the shape
 */
exports.removeShape = (shapeType, shapeGuid) => ({
    type: 'SN_DOCVIEWER_DOCUMENT_REMOVE_SHAPE',
    shapeType,
    shapeGuid,
});
/**
 * Action that updates the polling interval
 * @param pollInterval The interval in millisecs
 */
exports.setPollInterval = (pollInterval) => ({
    type: 'SN_DOCVIEWER_DOCUMENT_SET_POLL_INTERVAL',
    pollInterval,
});
/**
 * Action that updates the store with the given permission values
 * @param canEdit 'Can edit' permission value
 * @param canHideRedaction 'Can hide redaction' permission value
 * @param canHideWatermark 'Can hide watermark' permission value
 */
exports.documentPermissionsReceived = (canEdit, canHideRedaction, canHideWatermark) => ({
    type: 'SN_DOCVEWER_DOCUMENT_PERMISSIONS_RECEIVED',
    canEdit,
    canHideRedaction,
    canHideWatermark,
});
/**
 * Action that will be fired when a save request has been sent
 */
exports.saveChangesRequest = () => ({
    type: 'SN_DOCVEWER_DOCUMENT_SAVE_CHANGES_REQUEST',
});
/**
 * Action that will be fired on save error
 * @param error The error value
 */
exports.saveChangesError = (error) => ({
    type: 'SN_DOCVEWER_DOCUMENT_SAVE_CHANGES_ERROR',
    error,
});
/**
 * Action that will be fired when saving succeeded
 */
exports.saveChangesSuccess = () => ({
    type: 'SN_DOCVEWER_DOCUMENT_SAVE_CHANGES_SUCCESS',
});
/**
 * Action that rotates the specified shapes for the given page(s)
 * @param pages The pages to rotate
 * @param degree The rotation angle in degrees
 */
exports.rotateShapesForPages = (pages, degree) => ({
    type: 'SN_DOCVEWER_DOCUMENT_ROTATE_SHAPES_FOR_PAGES',
    pages,
    degree,
});
/**
 * Thunk action to call the Save endpoint with the current document state to save changes
 */
exports.saveChanges = () => {
    return (dispatch, getState, api) => __awaiter(this, void 0, void 0, function* () {
        dispatch(exports.saveChangesRequest());
        try {
            yield api.saveChanges(getState().sensenetDocumentViewer.documentState.document, getState().sensenetDocumentViewer.previewImages.AvailableImages);
            dispatch(exports.saveChangesSuccess());
        }
        catch (error) {
            dispatch(exports.saveChangesError(error));
        }
    });
};
/**
 * helper method to apply shape rotations
 * @param shapes the shape(s) to rotate
 * @param degree the rotation angle in degrees
 * @param pages the page info
 */
exports.applyShapeRotations = (shapes, degree, pages) => ([
    ...shapes.map((s) => {
        const page = pages.find((p) => p.index === s.imageIndex);
        if (page) {
            const angle = (Math.PI / 180) * services_1.ImageUtil.normalizeDegrees(degree);
            const [sin, cos] = [Math.sin(angle), Math.cos(angle)];
            const oldX = s.x - page.size.height / 2;
            const oldY = s.y - page.size.width / 2;
            const newX = oldX * cos - oldY * sin;
            const newY = oldY * cos + oldX * sin;
            return Object.assign({}, s, { x: newX + page.size.height / 2, y: newY + page.size.width / 2 });
        }
        return s;
    }),
]);
/**
 * The default state data for the Document
 */
exports.defaultState = {
    document: {
        hostName: '',
        shapes: {
            annotations: [],
            highlights: [],
            redactions: [],
        },
        documentName: '',
        documentType: '',
        fileSizekB: 0,
        idOrPath: 0,
        pageAttributes: [],
        pageCount: -1,
    },
    error: undefined,
    isLoading: true,
    idOrPath: undefined,
    version: undefined,
    canEdit: false,
    hasChanges: false,
    canHideRedaction: false,
    canHideWatermark: false,
    pollInterval: 2000,
};
/**
 * Reducer method for the document state
 * @param state the current state
 * @param action the action to dispatch
 */
exports.documentStateReducer = (state = exports.defaultState, action) => {
    switch (action.type) {
        case 'SN_DOCVIEWER_DOCUMENT_RESET_DOCUMENT':
            return Object.assign({}, exports.defaultState);
        case 'SN_DOCVIEWER_DOCUMENT_DATA_RECEIVED':
            return Object.assign({}, state, { document: Object.assign({}, state.document, action.document), error: undefined, isLoading: false, hasChanges: false });
        case 'SN_DOCVIEWER_DOCUMENT_DATA_RECEIVE_ERROR':
            return Object.assign({}, state, { document: Object.assign({}, state.document, { pageCount: 0 }), error: action.error, isLoading: false });
        case 'SN_DOCVIEWER_DOCUMENT_UPDATE_SHAPE':
            return Object.assign({}, state, { hasChanges: true, document: state.document && Object.assign({}, state.document, { shapes: Object.assign({}, state.document && state.document.shapes, { [action.shapeType]: state.document && state.document.shapes && state.document.shapes[action.shapeType].map((shape) => {
                            if (shape.guid === action.shapeGuid) {
                                return action.shapeData;
                            }
                            return shape;
                        }).filter((shape) => shape !== undefined) }) }) });
        case 'SN_DOCVIEWER_DOCUMENT_REMOVE_SHAPE':
            return Object.assign({}, state, { hasChanges: true, document: state.document && Object.assign({}, state.document, { shapes: Object.assign({}, state.document && state.document.shapes, { [action.shapeType]: state.document && state.document.shapes && state.document.shapes[action.shapeType]
                            .filter((shape) => shape && shape.guid !== action.shapeGuid) }) }) });
        case 'SN_DOCVEWER_DOCUMENT_ROTATE_SHAPES_FOR_PAGES':
            return Object.assign({}, state, { hasChanges: true, document: state.document && Object.assign({}, state.document, { shapes: {
                        annotations: exports.applyShapeRotations(state.document.shapes.annotations, action.degree, action.pages),
                        highlights: exports.applyShapeRotations(state.document.shapes.highlights, action.degree, action.pages),
                        redactions: exports.applyShapeRotations(state.document.shapes.redactions, action.degree, action.pages),
                    } }) });
        case 'SN_DOCVEWER_DOCUMENT_PERMISSIONS_RECEIVED':
            return Object.assign({}, state, { canEdit: action.canEdit, canHideRedaction: action.canHideRedaction, canHideWatermark: action.canHideWatermark });
        case 'SN_DOCVEWER_DOCUMENT_SAVE_CHANGES_SUCCESS':
            return Object.assign({}, state, { hasChanges: false });
        case 'SN_DOCVIEWER_DOCUMENT_SET_POLL_INTERVAL':
            return Object.assign({}, state, { pollInterval: action.pollInterval });
        default:
            return state;
    }
};
//# sourceMappingURL=Document.js.map