"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const services_1 = require("../services");
/**
 * Action that updates the store when the preview image fetching starts
 * @param documentData the provided document data
 */
exports.getAvailabelImagesAction = (documentData) => ({
    type: 'SN_DOCVIEWER_PREVIEWS_GET_IMAGES',
    documentData,
});
/**
 * Action that updates the store with the received image data
 * @param imageData the received image data
 */
exports.availabelImagesReceivedAction = (imageData) => ({
    type: 'SN_DOCVIEWER_PREVIEWS_IMAGES_RECEIVED',
    imageData,
});
/**
 * Action that updates the store with an error message when a fetching error happens
 * @param error The error message
 */
exports.availabelImagesReceiveErrorAction = (error) => ({
    type: 'SN_DOCVIEWER_PREVIEWS_IMAGES_RECEIVE_ERROR',
    error,
});
/**
 * Action that rotates the pages with the provided index(es) with a given angle
 * @param imageIndexes The image indexes to rotate
 * @param amount The rotation amount in degrees
 */
exports.rotateImages = (imageIndexes, amount) => ({
    type: 'SN_DOCVIEWER_PREVIEWS_IMAGES_ROTATE',
    imageIndexes,
    amount,
});
/**
 * Thunk action that fetches the available image info from the provided API endpoint
 * @param documentData
 * @param version
 */
exports.getAvailableImages = (documentData, version = 'V1.0A') => {
    return (dispatch, getState, api) => __awaiter(this, void 0, void 0, function* () {
        dispatch(exports.getAvailabelImagesAction(documentData));
        let docData;
        try {
            const state = getState();
            docData = yield api.getExistingPreviewImages(documentData, version, state.sensenetDocumentViewer.viewer.showWatermark);
        }
        catch (error) {
            dispatch(exports.availabelImagesReceiveErrorAction(error.message || Error('Error getting preview images')));
            return;
        }
        dispatch(exports.availabelImagesReceivedAction(docData));
    });
};
/**
 * Action that updates the store with the provided document data
 * @param documentData
 * @param version
 * @param page
 */
exports.previewAvailableAction = (documentData, version, page) => ({
    type: 'SN_DOCVIEWER_PREVIEWS_PREVIEW_AVAILABLE',
    documentData,
});
/**
 * Action that updates the store with the received image data
 * @param documentData
 * @param version
 * @param page
 * @param imageData
 */
exports.previewAvailableReceivedAction = (documentData, version, page, imageData) => ({
    type: 'SN_DOCVIEWER_PREVIEWS_PREVIEW_AVAILABLE_RECEIVED',
    documentData,
    version,
    page,
    imageData,
});
/**
 * Action that updates the state when no preview images are available
 */
exports.previewNotAvailableReceivedAction = () => ({
    type: 'SN_DOCVIEWER_PREVIEWS_PREVIEW_NOT_AVAILABLE_RECEIVED',
});
/**
 * Action that updates the state with an error message
 * @param error The error message
 */
exports.previewAvailableErrorAction = (error) => ({
    type: 'SN_DOCVIEWER_PREVIEWS_PREVIEW_AVAILABLE_ERROR',
    error,
});
/**
 * Action that sets the polling interval
 * @param pollInterval the new polling interval in millisecs
 */
exports.setPagePollInterval = (pollInterval) => ({
    type: 'SN_DOCVIEWER_DOCUMENT_PAGE_SET_POLL_INTERVAL',
    pollInterval,
});
/**
 * Thunk action that fetches the available images from the provided API endpoint
 * @param documentData
 * @param version
 * @param page
 */
exports.previewAvailable = (documentData, version = 'V1.0A', page = 1) => {
    return (dispatch, getState, api) => __awaiter(this, void 0, void 0, function* () {
        dispatch(exports.previewAvailableAction(documentData, version, page));
        let docData;
        try {
            const state = getState();
            docData = yield api.isPreviewAvailable(documentData, version, page, state.sensenetDocumentViewer.viewer.showWatermark);
        }
        catch (error) {
            dispatch(exports.availabelImagesReceiveErrorAction(error.message || Error(`Error getting preview image for page ${page}`)));
            return;
        }
        if (docData) {
            dispatch(exports.previewAvailableReceivedAction(documentData, version, page, docData));
        }
        else {
            dispatch(exports.previewNotAvailableReceivedAction);
        }
    });
};
/**
 * Reducer for the preview images
 * @param state the current state
 * @param action the dispatched action
 */
exports.previewImagesReducer = (state = { AvailableImages: [], error: null, hasChanges: false, pollInterval: 2000 }, action) => {
    const actionCasted = action;
    switch (actionCasted.type) {
        case 'SN_DOCVIEWER_PREVIEWS_GET_IMAGES':
            return Object.assign({}, state, { AvailableImages: [], error: null });
        case 'SN_DOCVIEWER_PREVIEWS_IMAGES_RECEIVED':
            return Object.assign({}, state, { hasChanges: false, AvailableImages: action.imageData });
        case 'SN_DOCVIEWER_PREVIEWS_IMAGES_RECEIVE_ERROR':
            return Object.assign({}, state, { AvailableImages: [], error: action.error });
        case 'SN_DOCVIEWER_PREVIEWS_IMAGES_ROTATE':
            return Object.assign({}, state, { hasChanges: true, AvailableImages: state.AvailableImages.map((img) => {
                    const newImg = Object.assign({}, img);
                    if (action.imageIndexes.indexOf(newImg.Index) >= 0) {
                        const newAngle = services_1.ImageUtil.normalizeDegrees((newImg.Attributes && newImg.Attributes.degree || 0) + (action.amount % 360)) % 360;
                        newImg.Attributes = Object.assign({}, newImg.Attributes, { degree: newAngle });
                    }
                    return newImg;
                }) });
        case 'SN_DOCVIEWER_PREVIEWS_PREVIEW_AVAILABLE_RECEIVED':
            return Object.assign({}, state, { AvailableImages: state.AvailableImages.map((img) => {
                    if (img.Index === action.page) {
                        return Object.assign({ Index: img.Index }, action.imageData);
                    }
                    return img;
                }) });
        case 'SN_DOCVEWER_DOCUMENT_SAVE_CHANGES_SUCCESS':
            return Object.assign({}, state, { hasChanges: false });
        case 'SN_DOCVIEWER_DOCUMENT_PAGE_SET_POLL_INTERVAL':
            return Object.assign({}, state, { pollInterval: actionCasted.pollInterval });
        default:
            return state;
    }
};
//# sourceMappingURL=PreviewImages.js.map